/*
 * Copyright (c) 2010 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

/**
 * Declares that the parameter or receiver to which this annotation is applied
 * does not receive any new aliases during execution of the method or
 * constructor. That is, {@link Unique} values can be safely passed to the
 * parameter or used as the receiver with the guarantee that they will still be
 * unique when the method returns.
 * <p>
 * Annotating <code>&#64;Borrowed("this")</code> on a constructor is defined to
 * be equivalent to annotating <code>&#64;Unique("return")</code>. Either of
 * these annotations indicates that the object being constructed is not aliased
 * during construction, which implies that the reference "returned" by the
 * {@code new} expression that invokes the constructor is unique. Which
 * annotation is preferred, <code>&#64;Borrowed("this")</code> or
 * <code>&#64;Unique("return")</code>, is a matter of programmer preference.
 * 
 * <p>
 * It is a modeling error to use this annotation on a parameter whose type is
 * primitive, e.g., <code>public void setValue(&#64;Borrowed int value)</code>
 * would generate a modeling error. It is a modeling error to annotate
 * <code>&#64;Borrowed("this")</code> on a {@code static} method, e.g.,
 * <code>&#64;Borrowed("this") public static void process()</code> would
 * generate a modeling error.
 * 
 * <h3>Semantics:</h3>
 * 
 * <p>
 * The <em>object</em>, if any, referenced by the annotated parameter at the
 * start of the method's execution is not returned by the method nor is it
 * assigned to a field of any object.
 * 
 * <h3>Examples:</h3>
 * 
 * In the below example, the first parameter to the {@code static} method
 * {@code run} promises that it will not hold onto an alias to the passed
 * {@code Cart} object. The two methods {@code go} and {@code log} promise that
 * they will not hold onto an alias to the receiver. The combination of these
 * three promises allows the model to be verified.
 * 
 * <pre>
 * public class Cart {
 * 
 *   static void run(@Borrowed Cart cart) {
 *     cart.go(10);
 *     cart.log(&quot;started moving&quot;);
 *     cart.go(20);
 *     cart.log(&quot;moved again&quot;);
 *   }
 * 
 *   int x, y;
 *   final List&lt;String&gt; log = new ArrayList&lt;String&gt;();
 * 
 *   &#064;Borrowed(&quot;this&quot;)
 *   void go(int value) {
 *     x += value;
 *     y += value;
 *   }
 * 
 *   &#064;Borrowed(&quot;this&quot;)
 *   void log(String msg) {
 *     log.add(msg);
 *   }
 *   ...
 * }
 * </pre>
 * 
 * This annotation is often used to support a {@link RegionLock} assertion on a
 * constructor because if the receiver is not leaked during object construction
 * then the state under construction will remain within the thread that invoked
 * {@code new}.
 * 
 * <pre>
 * &#064;RegionLock(&quot;Lock is this protects Instance&quot;)
 * public class Example {
 * 
 *   int x = 1;
 *   int y;
 * 
 *   &#064;Borrowed(&quot;this&quot;)
 *   public Example(int y) {
 *     this.y = y;
 *   }
 *   ...
 * }
 * </pre>
 * 
 * The scoped promise {@link Promise} can be used if the constructor is implicit
 * (i.e., generated by the compiler). It has the ability to place promises on
 * implicit and explicit constructors.
 * 
 * <pre>
 * &#064;RegionLock(&quot;Lock is this protects Instance&quot;)
 * &#064;Promise(&quot;@Borrowed(this) for new(**)&quot;)
 * public class Example {
 *   int x = 1;
 *   int y = 1;
 *   ...
 * }
 * </pre>
 * 
 * <h3>Javadoc usage notes:</h3>
 * 
 * This annotation may placed in Javadoc, which can be useful for Java 1.4 code
 * which does not include language support for annotations, via the
 * <code>&#064;annotate</code> tag. One complication is that the parameter being
 * annotated must be explicitly specified because the annotation can no longer
 * appear in the context of the parameter declaration.
 * 
 * <pre>
 * /**
 *  * @annotate Borrowed(&quot;a, b, c&quot;)
 *  &#42;/
 * public void m1(Object a, Object b, Object c) { ... }
 * </pre>
 * 
 * This annotation states that the three parameters are borrowed. Alternatively,
 * you can use several annotations as shown below.
 * 
 * <pre>
 * /**
 *  * @annotate Borrowed(&quot;a&quot;)
 *  * @annotate Borrowed(&quot;b&quot;)
 *  * @annotate Borrowed(&quot;c&quot;)
 *  &#42;/
 * public void m1(Object a, Object b, Object c) { ... }
 * </pre>
 * 
 * @see Unique
 */
@Documented
@Target( { ElementType.CONSTRUCTOR, ElementType.METHOD, ElementType.PARAMETER })
public @interface Borrowed {
  /**
   * When annotating a constructor or a method, this attribute must be {@code
   * "this"} to clarify the intent that it is the receiver that is borrowed. It
   * is a modeling error if the attribute is not {@code "this"} in this case.
   * 
   * <pre>
   * class C {
   *   &#064;Borrowed(&quot;this&quot;)
   *   public C() { ... }
   * 
   *   &#064;Borrowed(&quot;this&quot;)
   *   void method() { ... }
   *   ...
   * }
   * </pre>
   * 
   * This attribute is not used when annotating a parameter; it is a modeling
   * error for the value to be anything other than the default when annotating a
   * parameter.
   * 
   * <pre>
   * int proc(@Borrowed int value       /* Illegal: Parameter has primitive type &#42;/, 
   *          &#064;Borrowed Object settings /* Legal: Parameter has reference type &#42;/) { ... }
   * </pre>
   * 
   * The value of this attribute must conform to the following grammar (in <a
   * href="http://www.ietf.org/rfc/rfc4234.txt">Augmented Backus&ndash;Naur
   * Form</a>):
   * 
   * <pre>
   * value = [&quot;this&quot;] ; See above comments
   * </pre>
   */
  String value() default "";
}
