/*
 * Copyright (c) 2010 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

/**
 * The class to which this annotation is applied is declared to be containable.
 * That is, instances of the class can be safely encapsulated via region
 * aggregation into other objects as methods of this class do not leak
 * references to themselves or any of the other objects that they transitively
 * reference.
 * 
 * <p>While this property could be assured in a variety ways, JSure currently
 * assures it by requiring the fields and methods of the annotated type to be 
 * themselves annotated in a certain way:
 * 
 * <ul>
 *   <li>Each non-<code>static</code> method must be annotated 
 *       <code>&#064;Borrowed("this")</code>.
 *   <li>Each constructor must be annotated <code>&#064;Unique("return")</code>.
 *   <li>Each non-primitively typed field must be
 *      <ul>
 *         <li>Of a type annotated <code>&#064;Containable</code>.
 *         <li>Annotated <code>&#064;Unique</code>.
 *         <li>Annotated <code>&#064;Aggregate</code> or <code>&#064;AggregateInRegion</code>.
 *       </ul>
 * </ul>
 * 
 * <h3>Semantics:</h3>
 * 
 * For each object transitively referenced by the annotated instance
 * (including the instance itself), no reference to that object is
 * returned by any method/constructor of the class nor is that object
 * referenced by a field of any other object that is not also transitively
 * referenced by the instance (including the instance itself).
 * 
 * <h3>Examples:</h3>
 * 
 * <p>The class <code>Point</code> is a containable class that uses only primitively
 * typed fields.  Instances of it are in turn contained by the <code>Rectangle</code>
 * class.
 * 
 * <pre>
 * &#064;Containable
 * public class Point {
 *   private int x;
 *   private int y;
 *   
 *   &#064;Unique(&quot;return&quot;)
 *   &#064;RegionEffects(&quot;none&quot;)
 *   public Point(int x, int y) {
 *     this.x = x;
 *     this.y = y;
 *   }
 *   
 *   &#064;Borrowed(&quot;this&quot;)
 *   &#064;RegionEffects(&quot;writes Instance&quot;)
 *   public void translate(int dx, int dy) {
 *     x += dx;
 *     y += dy;
 *   }
 * }
 * 
 * &#064;Containable
 * public class Rectangle {
 *   &#064;Unique
 *   &#064;Aggregate
 *   private final Point topLeft;
 * 
 *   &#064;Unique
 *   &#064;Aggregate
 *   private final Point bottomRight;
 *   
 *   &#064;Unique(&quot;return&quot;)
 *   public Rectangle(int x1, int y1, int x2, int y2) {
 *     topLeft = new Point(x1, y1);
 *     bottomRight = new Point(x2, y2);
 *   }
 *   
 *   &#064;Borrowed(&quot;this&quot;)
 *   &#064;RegionEffects(&quot;writes Instance&quot;)
 *   public void translate(int dx, int dy) {
 *     topLeft.translate(dx, dy);
 *     bottomRight.translate(dx, dy);
 *   }
 * }
 * </pre>
 * 
 * <h3>Javadoc usage notes:</h3>
 * 
 * This annotation may placed in Javadoc, which can be useful for Java 1.4 code
 * which does not include language support for annotations, via the
 * <code>&#064;annotate</code> tag.
 * 
 * <pre>
 * /**
 *  * &#064;annotate Containable
 *  *&#047;
 * public class Point {
 *   ...
 * }
 * </pre>
 * 
 * @see ThreadSafe
 */
@Documented
@Target(ElementType.TYPE)
public @interface Containable {
	// Marker annotation
}