/*
 * Copyright (c) 2005 Brian Goetz and Tim Peierls
 * Released under the Creative Commons Attribution License
 *   (http://creativecommons.org/licenses/by/2.5)
 * Official home: http://www.jcip.net
 *
 * Any republication or derived work distributed in source code form
 * must include this copyright and license notice.
 * 
 * 
 * Copyright (c) 2010 SureLogic, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.surelogic;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;

/**
 * The class to which this annotation is applied is thread-safe. This means that
 * no sequences of accesses (reads and writes to public fields, calls to public
 * methods) may put the object into an invalid state, regardless of the
 * interleaving of those actions by the runtime, and without requiring any
 * additional synchronization or coordination on the part of the caller. Even if
 * one or more {@link RegionLock} models has been developed to document the
 * locking policy of a class this annotation can help to clarify that the
 * overall class is thread-safe.
 * 
 * <p>
 * This annotation <em>does not</em> imply that a sequence made up of calls to
 * methods of this class or accesses to fields of this class are atomic. It is a
 * the responsibility of the caller to insure that such sequences execute
 * atomically
 * 
 * <p>
 * This annotation is currently assured, <em>but not defined</em>, by
 * restricting how the fields of the class are declared and annotated. This is a
 * very conservative, although easily understood, way to assure the class.
 * Specifically, for a class annotated as <code>&#064;ThreadSafe</code> to
 * assure, each field must be either
 * <ul>
 * <li>Declared <code>final</code> and be "safe."
 * <li>Declared <code>volatile</code> and be "safe."
 * <li>Protected by a lock (by being a member of a region associated with a lock
 * via a <code>&#064;RegionLock</code> annotation) and be "safe."
 * </ul>
 * where "safe" means one of the following conditions is true:
 * <ul>
 * <li>The declared type is primitive.
 * <li>The declared type is annotated <code>&#064;ThreadSafe</code>.
 * <li>The declared type is annotated <code>&#064;Containable</code> and the
 * field is
 * <ul>
 * <li>Annotated <code>&#064;Unique</code>.
 * <li>Annotated with <code>&#064;Aggregate</code> or
 * <code>&#064;AggregateInRegion</code> such that each destination region is
 * protected by a lock.
 * </ul>
 * </ul>
 * 
 * <p>
 * <i>Implementation note:</i> This annotation is derived from
 * <code>&#064;ThreadSafe</code> proposed by Brian Goetz and Tim Peierls in the
 * book <i>Java Concurrency in Practice</i> (Addison-Wesley 2006) we have simply
 * adapted it to have semantics as a promise. Further, the annotation in
 * {@code net.jcip.annotations} may be used instead of this one with the same
 * tool behavior. One difference between the two annotations is that the one in
 * {@code net.jcip.annotations} has retention policy of
 * {@link java.lang.annotation.RetentionPolicy#RUNTIME} while the one in
 * {@code com.surelogic} defaults to
 * {@link java.lang.annotation.RetentionPolicy#CLASS}.
 * 
 * <h3>Semantics:</h3>
 * 
 * All accesses to a non-<code>final</code>, non-<code>volatile</code> region
 * of an instance of the annotated class occur either when
 * <ul>
 *   <li>The instance is under construction and only accessible by a single
 *   thread&mdash;the one that invoked the constructor&mdash;or
 *   <li>The lock associated with the region is held by the thread making
 *   the access.
 * </ul>
 *   
 * <h3>Examples:</h3>
 * 
 * The <code>Point</code> class is thread-safe because its fields are
 * <code>final</code> and of primitive type.
 * 
 * <pre>
 * &#064;ThreadSafe
 * &#064;Containable
 * public class Point {
 *   private final int x;
 * 
 *   private final int y;
 * 
 *   &#064;Unique(&quot;return&quot;)
 *   &#064;RegionEffects(&quot;none&quot;)
 *   public Point(int x, int y) {
 *     this.x = x;
 *     this.y = y;
 *   }
 * 
 *   &#064;Borrowed(&quot;this&quot;)
 *   &#064;RegionEffects(&quot;none&quot;)
 *   &#064;Unique(&quot;return&quot;)
 *   public Point translate(int dx, int dy) {
 *     return new Point(x + dx, y + dy);
 *   }
 * }
 * </pre>
 * 
 * <p>
 * The <code>Rectangle</code> class is thread-safe because its fields are
 * <code>final</code> and of a type annotated <code>&#064;ThreadSafe</code>:
 * 
 * <pre>
 * &#064;ThreadSafe
 * &#064;Containable
 * public class Rectangle {
 *   &#064;Unique
 *   &#064;Aggregate
 *   private final Point topLeft;
 * 
 *   &#064;Unique
 *   &#064;Aggregate
 *   private final Point bottomRight;
 * 
 *   &#064;Unique(&quot;return&quot;)
 *   &#064;RegionEffects(&quot;none&quot;)
 *   public Rectangle(@Unique Point a, @Unique Point b) {
 *     topLeft = a;
 *     bottomRight = b;
 *   }
 * 
 *   &#064;Borrowed(&quot;this&quot;)
 *   &#064;RegionEffects(&quot;none&quot;)
 *   &#064;Unique(&quot;return&quot;)
 *   public Rectangle translate(int dx, int dy) {
 *     return new Rectangle(topLeft.translate(dx, dy), bottomRight.translate(dx,
 *         dy));
 *   }
 * }
 * </pre>
 * 
 * <p>
 * Alternatively, we can create a mutable thread-safe <code>Rectangle</code>
 * class by protecting the state with a lock, and aggregating a containable
 * mutable, but not thread-safe, <code>Point</code> class:
 * 
 * <pre>
 * &#064;Containable
 * public class Point {
 *   private int x;
 *   private int y;
 *   
 *   &#064;Unique(&quot;return&quot;)
 *   &#064;RegionEffects(&quot;none&quot;)
 *   public Point(int x, int y) {
 *     this.x = x;
 *     this.y = y;
 *   }
 *   
 *   &#064;Borrowed(&quot;this&quot;)
 *   &#064;RegionEffects(&quot;writes Instance&quot;)
 *   public void translate(int dx, int dy) {
 *     x += dx;
 *     y += dy;
 *   }
 * }
 * 
 * &#064;ThreadSafe
 * &#064;RegionLock(&quot;Lock is this protects Instance&quot;)
 * public class Rectangle {
 *   &#064;Unique
 *   &#064;Aggregate
 *   private final Point topLeft;
 * 
 *   &#064;Unique
 *   &#064;Aggregate
 *   private final Point bottomRight;
 * 
 *   &#064;Unique(&quot;return&quot;)
 *   public Rectangle(int x1, int y1, int x2, int y2) {
 *     topLeft = new Point(x1, y1);
 *     bottomRight = new Point(x2, y2);
 *   }
 *   
 *   &#064;Borrowed(&quot;this&quot;)
 *   &#064;RegionEffects(&quot;writes Instance&quot;)
 *   public synchronized void translate(int dx, int dy) {
 *     topLeft.translate(dx, dy);
 *     bottomRight.translate(dx, dy);
 *   }
 * }
 * </pre>
 * 
 * <h3>Javadoc usage notes:</h3>
 * 
 * This annotation may placed in Javadoc, which can be useful for Java 1.4 code
 * which does not include language support for annotations, via the
 * <code>&#064;annotate</code> tag.
 * 
 * <pre>
 * /**
 *  * &#064;annotate ThreadSafe
 *  * &#064;annotate RegionLock(&quot;Lock is this protects Instance&quot;)
 *  *&#047;
 * public class Rectangle {
 *   ...
 * }
 * </pre>
 * 
 * @see Containable
 * @see Immutable
 * @see NotThreadSafe
 * @see RegionLock
 */
@Documented
@Target(ElementType.TYPE)
public @interface ThreadSafe {
	// Marker annotation
}